import logging
import signal
import subprocess
from time import sleep

import mysql.connector as mariadb

import time
import spidev
from RPi import GPIO

log = logging.getLogger(__name__)

running = True

MASTER_NAME = 'w1_bus_master1'
BASE_PATH = '/sys/bus/w1/devices'


class Output:
    def __init__(self, pin):
        self.pin = pin
        GPIO.setup(self.pin, GPIO.OUT, initial=GPIO.HIGH)

    def on(self):
        GPIO.output(self.pin, GPIO.HIGH)

    def off(self):
        GPIO.output(self.pin, GPIO.LOW)

    def control(self, state):
        if state == 0:
            self.on()
        else:
            self.off()


class MCP3008:
    """Class representing an MCP3008 ADC"""
    def __init__(self, bus=0, device=0):
        """
        Initialize SPI device for MCP3008
        :param bus: SPI bus (usually 0)
        :param device: SPI slave (SS/CS/CE)
        """
        self.spi = spidev.SpiDev(bus, device)
        self.spi.open(0, 0)
        self.spi.max_speed_hz = 10000

    def read_channel(self, ch=0):
        """
        Perform measurement on an ADC channel
        :param ch: ADC channel (0-7)
        :return: 10-bit integer measurement (0-1023)
        """
        to_send = [0x01, 0x80, 0x00]
        received = self.spi.xfer2(to_send)
        x = (received[1] & 3) << 8 | received[2]
        return x

    def __del__(self):
        self.spi.close()


def save_readings_value(sensor_value, sensor_type):
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("""INSERT INTO readings (readingDate,value,sensorID) VALUES (%s,%s,%s)""", (time.strftime('%Y-%m-%d %H:%M:%S'),sensor_value, sensor_type))
        conn.commit()
        log.debug("Saved sensor value to database")
        return True
    except Exception as e:
        log.exception("DB update failed: {!s}".format(e))


def read_last_temperature_value():
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("""SELECT value FROM readings WHERE sensorID=1 ORDER BY readingDate DESC LIMIT 1;""")
        read_data = cursor.fetchall()
        raw_data = read_data[0][0]
        log.debug("Reading last temperature value")
        return raw_data
    except Exception as e:
        log.exception("DB reading failed: {!s}".format(e))


def read_last_moisture_value():
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("""SELECT value FROM readings WHERE sensorID=2 ORDER BY readingDate DESC LIMIT 1;""")
        read_data = cursor.fetchall()
        raw_data = read_data[0][0]
        log.debug("Reading last moisture value")
        return raw_data
    except Exception as e:
        log.exception("DB reading failed: {!s}".format(e))


def read_trimmer(device=0):
    """Read a value from CH0 on MCP3008 slave <device> and pri  nt it to console.
    :param device: SPI slave (SS/CE/CS: 0 or 1 for standard RPi 3)
    """
    while True:
        spi = spidev.SpiDev()
        spi.open(0, 0)
        spi.max_speed_hz = 10000

        to_send = [0x01, 0x80, 0x00]
        received = spi.xfer2(to_send)
        x = (received[1] & 3) << 8 | received[2]
        return x


def value_to_percent(value):
    """Convert a 10-bit measurement to a percentage
    :param value: ADC measurement (0-1023)
    :return: Percentage with 1 decimal (0.0 - 100.0)
    """
    startvalue = read_trimmer()
    return int(startvalue / 10.23)


def get_slave_id():
    # Getting sensor name
    path1 = BASE_PATH + '/' + MASTER_NAME + '/' + 'w1_master_slaves'
    slavelist = []
    myfile1 = open(path1)
    for line in myfile1:
        slavelist.append(line.strip('\n'))
    myfile1.close()
    return slavelist[0]


def get_temperature():
    # open het 'w1_slave' bestand voor de sensor:
    path2 = BASE_PATH + '/' + get_slave_id() + '/' + 'w1_slave'
    with open(path2, 'r') as f:
        output = f.read()
    assert f.closed
    # haal er er de juiste regel uit:
    keyword = ' t='
    num1 = output.find(keyword)
    num2 = num1 + 3
    # haal de waarde van t uit de regel:
    value = output[num2:]
    # zet om naar int en deel door 1000 alvorens de waarde te returnen
    float_value = float(value)/1000
    temp = round(float_value, 2)
    return temp


def compare_temperature(max_temp):
    current_temperature = read_last_temperature_value()
    if current_temperature > max_temp:
        return write_fan_state(True)
    else:
        return write_fan_state(False)


def compare_moisture():
    current_moisture = read_last_moisture_value()
    if int(current_moisture) > 30:
        return write_pump_state(True)
    else:
        return write_pump_state(False)


def read_in_temperature():
    log.debug('Measuring Temperature')
    temp_value = get_temperature()
    save_readings_value(temp_value, 1)


def read_in_moisture():
    log.debug('Measuring Soil Moisture')
    moist_value = value_to_percent(moistureSensor.read_channel)
    save_readings_value(moist_value, 2)


def write_fan_state(state):
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("UPDATE sensorsActuators SET actuatorState=" + str(state) + " WHERE sensorType='fan';")
        conn.commit()
        log.debug("Saved fan state to database")
        return True
    except Exception as e:
        log.exception("DB update failed: {!s}".format(e))


def write_pump_state(state):
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("UPDATE sensorsActuators SET actuatorState=" + str(state) + " WHERE sensorType='pump';")
        conn.commit()
        log.debug("Saved pump state to database")
        return True
    except Exception as e:
        log.exception("DB update failed: {!s}".format(e))


def control_pump():
    log.debug('Controlling Pump')
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("""SELECT actuatorState FROM sensorsActuators WHERE sensorType='pump' LIMIT 1;""")
        read_data = cursor.fetchall()
        raw_data = read_data[0][0]
        if raw_data == 1:
            pump.control(1)
            time.sleep(20)
            pump.control(0)
            write_pump_state(0)
        else:
            pump.control(0)
    except Exception as e:
        log.exception("Failed reading pump status: {!s}".format(e))


def control_fan():
    # Read database if fan should be on or off and controls it that way.
    log.debug('Controlling Fan')
    try:
        conn = mariadb.connect(database='project1', user='project1-sensor', password='07listenjourneylabor52')
        cursor = conn.cursor()
        cursor.execute("""SELECT actuatorState FROM sensorsActuators WHERE sensorType='fan' LIMIT 1;""")
        read_data = cursor.fetchall()
        raw_data = read_data[0][0]
        fan.control(raw_data)
    except Exception as e:
        log.exception("Failed reading fan status: {!s}".format(e))


def setup():
    def shutdown(*args):
        global running
        running = False
        # try:
        #     sys.exit(0)
        # except KeyError as ex:
        #     pass

    signal.signal(signal.SIGTERM, shutdown)


def loop():
    count = subprocess.check_output("/bin/ps aux | wc -l", shell=True)
    save_readings_value("process_count", count)
    sleep(5)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    GPIO.setmode(GPIO.BCM)
    try:
        setup()
        fan = Output(14)
        pump = Output(15)

        maxTemp = 27

        moistureSensor = MCP3008()

        print("The max Temperature is: " + str(maxTemp))
        print("----------------------- Starting Program -----------------------")
        while True:
            for i in range(0, 3600, 1):
                read_in_temperature()
                read_in_moisture()
                if (i % 60) == 0:
                    compare_temperature(maxTemp)
                    if i == 0:
                        compare_moisture()
                control_fan()
                control_pump()
                time.sleep(1)

    except KeyboardInterrupt:
        pass
    finally:
        GPIO.cleanup()
